use errors::Result;
use imap_store::{IMAPConfig, IMAPStore};
use mail_dir_store::{MailDirConfig, MailDirStore};
use mail_store_state::MailStoreState;
use message::{Message, Uid};
use message_threads::MessageThreader;
use serde_json;
use std::fs;
use std::path::PathBuf;

#[derive(Debug, PartialEq)]
pub struct Name {
    pub delimiter: String,
    pub name: String,
}

#[derive(Deserialize)]
pub enum MailStoreConfig {
    IMAP(IMAPConfig),
    MailDir(MailDirConfig),
}

pub trait MailStore {
    fn is_folder_complete(&self, folder: &str, state: &MailStoreState) -> bool;
    fn list_folders(&mut self) -> Result<Vec<Name>>;
    fn fetch(&mut self, folder: &str, uid: Uid) -> Result<Message>;
    fn fetch_headers(&mut self, folder: &str, threads: &mut MessageThreader) -> Result<()>;
}

pub fn create_mail_store(path: &PathBuf) -> Result<Box<MailStore>> {
    let contents = fs::read_to_string(path)?;
    let config: MailStoreConfig = serde_json::from_str(&contents)?;
    Ok(match config {
        MailStoreConfig::IMAP(config) => Box::new(IMAPStore::new(config)),
        MailStoreConfig::MailDir(config) => Box::new(MailDirStore::new(config)),
    })
}
