extern crate chrono;
#[macro_use]
extern crate error_chain;
extern crate imap;
extern crate imap_proto;
extern crate libc;
extern crate mailparse;
extern crate memchr;
extern crate native_tls;
extern crate nom;
extern crate rayon;
extern crate regex;
extern crate serde;
extern crate walkdir;
#[macro_use]
extern crate serde_derive;
extern crate serde_json;

mod errors {
    // Create the Error, ErrorKind, ResultExt, and Result types
    error_chain!{
        foreign_links {
            Io(::std::io::Error);
            Imap(::imap::error::Error);
            Tls(::native_tls::Error);
            Tls2(::native_tls::HandshakeError<::std::net::TcpStream>);
            Serde(::serde_json::Error);
            StripPrefixError(::std::path::StripPrefixError);
            Nom(::nom::ErrorKind);
            Chrono(::chrono::ParseError);
            MailParse(::mailparse::MailParseError);
        }
    }
}

mod imap_store;
mod implementation;
pub mod interface;
mod job_processor;
mod mail_dir_store;
mod mail_store;
mod mail_store_state;
mod message;
mod message_threads;
mod sorted_filtered_tree;
mod sync_tree;
mod tree;

use std::os::raw::c_char;
extern {
    fn main_cpp(app: *const c_char, config_file: *const c_char);
}

fn main() {
    use std::ffi::CString;
    let mut args = ::std::env::args();
    let app = CString::new(args.next().unwrap()).unwrap();
    let config_file = CString::new(args.next().unwrap_or_else(||panic!("Provide a file with mail configuration."))).unwrap();
    unsafe {
        main_cpp(app.as_ptr(), config_file.as_ptr());
    }
}
